package controllers;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import views.JFrame_ElevatorSpecification;
import interfaces.ElevatorSpecification;

/**
 * This behaviour can be tested by executing {@link simulations.graphical.Elevator_GUI_KeyboardController}<br>
 * A simple keyboard controller for an {@link ElevatorSpecification} object.<br>
 * To simulate a button press inside the elevator to go to floor <code> n </code> then
 * press the digit character <code> 'n' </code> on the keyboard<br>
 * To simulate a button press to request the elevator to go UP at floor <code> n </code> then
 * press the character <code> '&gt;' </code>  followed by the digit character <code> 'n' </code> on the keyboard<br>
 * To simulate a button press to request the elevator to go DOWN at floor <code> n </code> then
 * press the character <code> '&lt;' </code>  followed by the digit character <code> 'n' </code> on the keyboard<br><br>
 * 
 * Follows a standard model-view-controller (MVC) design pattern.
 * @author jpaulgibson
 *
 */
public class KeyboardButtonsForElevator implements KeyListener {
	
	/**
	 * The elevator model to which the button press state updates will be sent
	 */
	private ElevatorSpecification elevator;
	
	/**
	 * The view frame from which key presses that are detected will be processed
	 */
	JFrame_ElevatorSpecification frame;
	
	/**
	 * Remembers if the previous key press was to go up
	 */
	boolean upPressed = false;
	
	/**
	 * Remembers if the previous key press was to go down
	 */
	boolean downPressed = false;
	
	/**
	 * The key to press in order to make an up request at a floor (which is specified by the next key press)
	 */
	public final static char UP_CHARACTER = '>';
	
	/**
	 * The key to press in order to make a down request at a floor (which is specified by the next key press)
	 */
	public final static char DOWN_CHARACTER = '<';
	
	/**
	 * Bind the controller to the view and model.<br>
	 * (so that the elevator reacts to the requests coming from the keyboard). <br>
	 * @param elevator is the model to be attached to the keyboard controller
	 * @param frame is the view which forwards the key presses associated to it
	 */
	public KeyboardButtonsForElevator(ElevatorSpecification elevator, JFrame_ElevatorSpecification frame ){
		
		this.elevator = elevator;
		this.frame = frame;
		frame.addKeyListener(this);
	}
	

	/**
     * To simulate a button press inside the elevator to go to floor <code> n </code> then
     * press the digit character 'n' on the keyboard<br>
     * To simulate a button press to request the elevator to go UP at floor <code> n </code> then
     * press the character '&gt;' followed by the digit character 'n' on the keyboard<br>
     * To simulate a button press to request the elevator to go DOWN at floor <code> n </code> then
     * press the character '&lt;'   followed by the digit character  'n'  on the keyboard<br>
     * We need only a single digit key (0..9) to specify the floor because the maximum number of floors is defined as 
     * {@link factoriesSPL.ElevatorFactory#MAXIMUM_NUMBER_OF_FLOORS}.<br>
     * An error message will report when an invalid floor is requested.<br>
     * 
	 */
	@Override
	public void keyTyped(KeyEvent e) {
		
	char c = e.getKeyChar();
		
	//Up request to precede a number input
	if (c =='>' ) {
		upPressed = true;
		downPressed = false;
		System.out.println("Which floor do you wish to go UP from?");
       
	//Down request to precede a number input
    } else if (c =='<' ) {
    	upPressed = false;
		downPressed = true;
		System.out.println("Which floor do you wish to go DOWN from?");
           
    }
	
	// Single digit keys for floor selection
    else if (c>='0' && c <= '9'){
		
		int floor = (int)c - 48;
		
		if (floor > elevator.topFloor()) System.out.println("This is not a valid floor");
            	
		else if (upPressed){
        	
        	if (floor < elevator.topFloor()){
            		
            		elevator.pressUp(floor);
            		System.out.println("Pressed UP button at floor "+ floor);
        	}
        	else System.out.println("This is not a valid floor");
        }
            	
       else if (downPressed) {
    	     if (floor != 0 && floor <= elevator.topFloor()){
            		
            		elevator.pressDown(floor);
            		System.out.println("Pressed DOWN button at floor "+ floor);
            	}
    	     else System.out.println("This is not a valid floor");
       }
       else {
            		elevator.pressFloorInElevator(floor);
            		System.out.println("Pressed floor button "+ floor+" inside the elevator");
            	}
            	
            	// any key other than an arrow up/down resets the values
            	upPressed = false;
        		downPressed = false;
                
        	  }
        }
        

	@Override
	public void keyPressed(KeyEvent e) {
		// Nothing to do 
		
	}
	@Override
	public void keyReleased(KeyEvent e) {
		// Nothing to do
		
	}
	
	

}
