package factoriesSPL;

import java.util.Random;

import models.elevatorLogicUnit.EmergencyLogicUnit;
import models.elevatorLogicUnit.ImprovedLogicUnit;
import models.elevatorLogicUnit.RandomMovementLogicUnit;
import models.elevatorLogicUnit.SmartLogicUnit;
import models.elevatorLogicUnit.StupidLogicUnit;
import models.elevators.Elevator;
import models.elevators.ElevatorWithDoors;
import models.elevators.ElevatorWithDoorsAndLockableFloors;
import models.elevators.ElevatorWithLockableFloors;
import enumerations.LogicType;
import enumerations.SecurityType;
import interfaces.LogicUnitForElevatorSpecification;
import interfaces.ElevatorSpecification;


/**
 * This class controls the different variations of elevator that we can construct.
 * In a SPL approach not all features/variations are permitted/valid/meaningful.
 * In this simple example, we have 168 different configurations:
 * <ul>
 * <li> Number of floors - 2 .. 10 (inclusive)
 * <li> Doors - YES/NO
 * <li> Number of controller types - RANDOM, STUPID, IMPROVED, EMERGENCY, SMART
 * <li> Security Type - NONE, OPTION1, OPTION2
 * </ul>
 * As new features and variations are added to the SPL, this class needs to be updated accordingly <br><br>
 * This factory is tested by {@link unit_tests.factoriesSPL.JUnit_ElevatorFactory} and by
 * {@link validation_tests.validation_factoriesSPL.Validation_ElevatorFactory}<br>
 * 
 * @author jpaulgibson
 */
public class ElevatorFactory {
	
	public static final int MAXIMUM_NUMBER_OF_FLOORS = ElevatorSpecification.MAXIMUM_NUMBER_OF_FLOORS;
	public static final int MINIMUM_NUMBER_OF_FLOORS = ElevatorSpecification.MINIMUM_NUMBER_OF_FLOORS;
	
	public static final int DEFAULT_NUMBER_OF_FLOORS= 4;
	
	public static final boolean DEFAULT_DOORS = true;
	public static final LogicType DEFAULT_CONTROLLER = LogicType.SMART;
	public static final SecurityType DEFAULT_SECURITY = SecurityType.NONE;
	
	public static final int DEFAULT_DELAY_TIME = 1000; // 1000 ms = 1 second;
	
	/**
	 * When choosing a random delay it must be no smaller than 1 second
	 */
	public static final int MINIMUM_RANDOM_DELAY = 1000; 
	
	/**
	 * When choosing a random delay it must be no bigger than 5 seconds
	 */
	public static final int MAXIMUM_RANDOM_DELAY = 5000; 
	
	/**
	 * Whether each floor is securely locked (or not) - not needed when there is no security
	 */
	public static boolean [] secure_floors;
	
	/**
	 * Tested by {@link unit_tests.factoriesSPL.JUnit_ElevatorFactory#test_makeElevator_default()}.<br>
	 * @return a DEFAULT elevator with the following configuration:
	 * <ul>
	 * <li> {@link #DEFAULT_NUMBER_OF_FLOORS}</li>
	 * <li> {@link #DEFAULT_DOORS}</li>
	 * <li> {@link #DEFAULT_CONTROLLER}</li>
	 * <li> {@link #DEFAULT_SECURITY}</li>
	 * <li> {@link #DEFAULT_DELAY_TIME}</li>
	 * </ul>
	 */
	public static ElevatorSpecification makeElevator(){
		
		return makeElevator(DEFAULT_DOORS,
				            DEFAULT_NUMBER_OF_FLOORS,
				            DEFAULT_CONTROLLER,
				            DEFAULT_SECURITY,
				            DEFAULT_DELAY_TIME);
	}
	

	/**
	 * Tested by {@link unit_tests.factoriesSPL.JUnit_ElevatorFactory#test_makeElevator_random()}.<br>
	 * @param rng is the random number generator used to generate the random configuration
	 * @return A randomly configured elevator where parameters are chosen using a discreet uniform distribution:
	 * <ul>
	 * <li> {@link #MINIMUM_NUMBER_OF_FLOORS} .. {@link #MAXIMUM_NUMBER_OF_FLOORS}</li>
	 * <li> {@link LogicType} </li>
	 * <li> {@link SecurityType} </li>
	 * <li>  <code> delay_time = </code>  {@link #MINIMUM_RANDOM_DELAY} ms .. {@link #MAXIMUM_RANDOM_DELAY} ms
	 * </ul>
	 */
	public static ElevatorSpecification makeElevator(Random rng){
		

		boolean doors = rng.nextBoolean();

		int number_of_floors = MINIMUM_NUMBER_OF_FLOORS + 
				               rng.nextInt(MAXIMUM_NUMBER_OF_FLOORS-MINIMUM_NUMBER_OF_FLOORS+1);
		
		int controller_index = rng.nextInt(LogicType.values().length);
		LogicType controller_type = LogicType.values()[controller_index];
		int security_index = rng.nextInt(SecurityType.values().length);
		SecurityType security_type = SecurityType.values()[security_index];
		
		int delay_time = (rng.nextInt(5)+1)*1000; //between 1 and 5 seconds
		
		return makeElevator(doors, number_of_floors, controller_type, security_type, delay_time);
	}
	
	/**
	 * Tested by {@link unit_tests.factoriesSPL.JUnit_ElevatorFactory#test_makeElevator()}.<br>
	 * The configuration of an observable elevator instance
	 * @param doors - are there doors on the floors (YES/NO)?
	 * @param number_of_floors - between 2 .. 8 (inclusive)
	 * @param controller_type - RANDOM, STUPID, IMPROVED, EMERGENCY
	 * @param security_type - NONE, OPTION1, OPTION2
	 * @param delay_time - the time between elevator updates that are observed by the observers
	 * @return the configured observable elevator
	 * @throws IllegalArgumentException if specified number of floors is not valid
	 */
	public static ElevatorSpecification makeElevator(boolean doors,
			                                         int number_of_floors, 
			                                         LogicType controller_type,
			                                         SecurityType security_type,
			                                         int delay_time)
	  throws IllegalArgumentException {
		
		if (number_of_floors < MINIMUM_NUMBER_OF_FLOORS || number_of_floors > MAXIMUM_NUMBER_OF_FLOORS)
			throw new IllegalArgumentException("The number of floors is not in the permitted range");
		
        LogicUnitForElevatorSpecification controller;
        
			switch (controller_type) {
		    case RANDOM:      
		    	controller = new RandomMovementLogicUnit(null);
		        break;
		    case STUPID:      
		    	controller = new StupidLogicUnit(null);
		        break;
		    case IMPROVED:      
		    	controller = new ImprovedLogicUnit(null);
		        break;
		    case SMART:      
		    	controller = new SmartLogicUnit(null);
		        break;
		    case EMERGENCY:      
		    	controller = new EmergencyLogicUnit(null);
		        break;
		    default: 
		    	controller = new EmergencyLogicUnit(null);
			 }
			
			ElevatorSpecification elevator = null;
			
			if (security_type == SecurityType.NONE) {
				
			    if (doors) elevator = new ElevatorWithDoors(number_of_floors,delay_time);
                      else elevator = new Elevator(number_of_floors, delay_time);
			}
			
			else{
				
				// By default, make all doors lockable
				boolean [] lockable = new boolean [number_of_floors];
				for (int floor=0; floor< number_of_floors; floor++)
					lockable [floor] = true; 
				
				if (security_type == SecurityType.OPTION1) {
					if (doors) elevator = new ElevatorWithDoorsAndLockableFloors(number_of_floors, lockable, SecurityType.OPTION1, delay_time);
	                else elevator = new ElevatorWithLockableFloors(number_of_floors, lockable, SecurityType.OPTION1, delay_time);
				}
				
				else if (security_type == SecurityType.OPTION2) {
					if (doors) elevator = new ElevatorWithDoorsAndLockableFloors(number_of_floors, lockable, SecurityType.OPTION2, delay_time);
	                else elevator = new ElevatorWithLockableFloors(number_of_floors, lockable, SecurityType.OPTION2, delay_time);
				}
				
			}
		
			elevator.installLogicUnit(controller);
	    	return elevator;
	}

}
