package factoriesSPL;

import interfaces.LogicUnitForElevatorSpecification;

import java.util.Random;

import models.elevatorLogicUnit.EmergencyLogicUnit;
import models.elevatorLogicUnit.ImprovedLogicUnit;
import models.elevatorLogicUnit.RandomMovementLogicUnit;
import models.elevatorLogicUnit.SmartLogicUnit;
import models.elevatorLogicUnit.StupidLogicUnit;
import enumerations.LogicType;

/**
* Tested by:
 * <ul> 
 * <li> unit tests -  {@link unit_tests.factoriesSPL.JUnit_LogicUnitFactory} </li>
 * <li> validation tests -  {@link validation_tests.validation_factoriesSPL.Validation_LogicUnitFactory} </li>
 * </ul>
 * Permits the creation of elevator logic units of different types as defined in {@link enumerations.LogicType}
 * and implemented as subclasses of {@link abstractions.LogcUnitForElevatorBinding} in package
 * {@link models.elevatorLogicUnit}.<br>
 * @author jpaulgibson
 *
 */
public class LogicUnitFactory {
	
	/**
	 * Tested by {@link unit_tests.factoriesSPL.JUnit_LogicUnitFactory#test_makeRandomController()}
	 * Generate a random logic unit
	 * @param rng is the random number generator used to pick one of the possible logic units
	 * @throws IllegalArgumentException if the random number generated parameter is <code> null </code>
	 * @return the randomly generated logic unit
	 */
	public static LogicUnitForElevatorSpecification makeLogicUnit(Random rng) throws IllegalArgumentException{
		
		if (rng == null) throw new IllegalArgumentException("The random number generator is null");
		
		int logic_unit_index = rng.nextInt(LogicType.values().length);
		LogicType logic_unit_type = LogicType.values()[logic_unit_index];
	
		return makeLogicUnit(logic_unit_type);
	}
	
	
	/**
	 * Generate a logic unit of the specified type<br>
	 * This method needs to be updated when new types of logic unit are added to the {@link enumerations.LogicType}<br>
	 * @param logic_unit_type is the type of controller to be constructed
	 * @return the logic unit constructed to match the logic unit type as specified
	 */
	public static LogicUnitForElevatorSpecification makeLogicUnit(LogicType logic_unit_type){
		
		LogicUnitForElevatorSpecification logic_unit = null;
		
		switch (logic_unit_type) {
	    case RANDOM:      
	    	logic_unit = new RandomMovementLogicUnit(null);
	        break;
	    case STUPID:      
	    	logic_unit = new StupidLogicUnit(null);
	        break;
	    case IMPROVED:      
	    	logic_unit = new ImprovedLogicUnit(null);
	        break;
	    case SMART:      
	    	logic_unit = new SmartLogicUnit(null);
	        break;
	    case EMERGENCY:      
	    	logic_unit = new EmergencyLogicUnit(null);
	        break;
	    default: // should not be required
	    	logic_unit = new EmergencyLogicUnit(null);
		 }
		return logic_unit;
	}

}
