package interfaces;

import enumerations.DirectionState;

/**
 * The interface offered by the elevator to the logic controller. <br>
 * This is a subset of the complete
 * interface of an elevator as specified by {@link interfaces.ElevatorSpecification}. <br>
 * With this interface we hide the methods specific to the elevator users which should
 * not be visible to the elevator logic unit.
 * @see interfaces.ElevatorForUsersSpecification
 * @author J Paul Gibson
 */
public interface ElevatorForLogicUnitSpecification {
	
	/**
	 * Permits elevator logic unit to see if a request to go down has been made at a specified floor
	 * @param floor specifies the floor at which the request is being checked
	 * @throws IllegalArgumentException if the floor is not valid (from 1.. NUMBER_OF_FLOORS-1)
	 * @return whether the down button is pressed at this floor
	 */
	public boolean downPressedAtFloor (int floor)throws IllegalArgumentException;
	
	/**
	 * Permits elevator logic unit to see if a request to go up has been made at a specified floor
	 * @param floor specifies the floor at which the request is being checked
	 * @throws IllegalArgumentException if the floor is not valid (from 0.. NUMBER_OF_FLOORS-2)
	 * @return whether the up button is pressed at this floor
	 */
	public boolean upPressedAtFloor (int floor)throws IllegalArgumentException;
	
	/**
	 * Permits elevator logic unit to see if there is a request to a particular floor
	 * @param floor specifies the floor at which the request is being checked
	 * @return Whether there is currently a request to go to the specified floor (inside the elevator)
     * @throws IllegalArgumentException if the number of floors specified is not in range
     */
	public boolean floorPressedInElevator (int floor) throws IllegalArgumentException;
	
	/**
	 * @param floor specifies the floor at which the request is being checked
	 * @return whether there are no requests for a floor (inside the elevator or at a floor) below the specified floor
     * @throws IllegalArgumentException if the number of floors specified is not in range
	 */
	public boolean noMoreRequestsBelow(int floor)throws IllegalArgumentException;
	
	/**
	 * @param floor specifies the floor at which the request is being checked
	 * @return whether are no requests for a floor (inside the elevator or at a floor) above the specified floor
     * @throws IllegalArgumentException if the number of floors specified is not in range
	 */
	public boolean noMoreRequestsAbove(int floor)throws IllegalArgumentException;
	
	/**
	 * @return the current direction in which the elevator is moving
	 */
    public DirectionState currentDirection ();
    
    /**
     * @return the current floor of the elevator
     */
    public int currentFloor ();
    
    /**
     * @return the number of the top floor
     */
    public int topFloor ();
    
	/**
     * Update the lift logic unit component - intended to be done by an engineer
	 * @param logic_unit is the new logic unit module that will decide the logic of the elevator movement
	 * @throws IllegalArgumentException if the logic_unit  is null
	 */
	public void installLogicUnit(LogicUnitForElevatorSpecification logic_unit) throws IllegalArgumentException;
    
}
