package interfaces;

import enumerations.DirectionState;

/**
 * The interface offered by the elevator to the users. <br>
 * This is a subset of the complete
 * interface of an elevator as specified by {@link interfaces.ElevatorSpecification}. <br>
 * With this interface we hide the methods specific to the elevator logic unit which should
 * not be visible to elevator users.
 * @see interfaces.ElevatorForLogicUnitSpecification
 * @author J Paul Gibson
 */
public interface ElevatorForUsersSpecification {
	
	/**
	 * Permits elevator users to request elevator to take them up
	 * @param floor specifies the floor at which the up request is made
	 * @throws IllegalArgumentException if the floor is not valid (from 0.. NUMBER_OF_FLOORS-2)
	 */
	public void pressUp(int floor)throws IllegalArgumentException;
	
	/**
	 * Permits elevator users to see if a request to go up has been made at a specified floor
	 * @param floor specifies the floor at which the request is being checked
	 * @throws IllegalArgumentException if the floor is not valid (from 0.. NUMBER_OF_FLOORS-2)
	 * @return if the up button is pressed on this floor
	 */
	public boolean upPressedAtFloor (int floor) throws IllegalArgumentException;
	
	/**
	 * Permits elevator users to request elevator to take them down
	 * @param floor specifies the floor at which the down request is made
	 * @throws IllegalArgumentException if the floor is not valid (from 1.. NUMBER_OF_FLOORS-1)
	 */
	public void pressDown(int floor)throws IllegalArgumentException;
	
	/**
	 * Permits elevator users to see if a request to go down has been made at a specified floor
	 * @param floor specifies the floor at which the request is being checked
	 * @throws IllegalArgumentException if the floor is not valid (from 1.. NUMBER_OF_FLOORS-1)
	 * @return if the down button is pressed on this floor
	 */
	public boolean downPressedAtFloor (int floor)throws IllegalArgumentException;
	
	
	/**
	 * Permits elevator users inside the elevator to request elevator to take them to a particular floor 
	 * @param floor specifies the floor at which the request is being checked
     * @throws IllegalArgumentException if the number of floors specified is not in range
	 */
	public void pressFloorInElevator (int floor) throws IllegalArgumentException;
	
	/**
	 * Permits elevator users inside the elevator to see if there is a request to a particular floor
	 * @return Whether there is currently a request to go to the specified floor (inside the elevator)
	 * @throws IllegalArgumentException if the number of floors specified is not in range
	 * @param floor specifies the floor at which the request is being checked
	 */
	public boolean floorPressedInElevator (int floor) throws IllegalArgumentException;
	
	/**
	 * Accessor (getter) function
	 * Permits elevator users to see the current floor of the elevator
	 * @return the current floor of the elevator
	 */
	public int currentFloor();
	
	/**
	 * Accessor (getter) function<br>
	 * Permits elevator users to see the current floor of the elevator
	 * @return the current floor of the elevator
	 */
	public DirectionState currentDirection();
	
	/**
	 * Accessor (getter) function<br>
	 * Permits elevator users to see the number of the top floor of the elevator
	 * @return the top floor of the elevator
	 */
	public int topFloor();

}
