package interfaces;

import enumerations.UserState;
import tools.HasInvariant;

/**
 * Tested by {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification}. <br>
 * For simulating users of an elevator<br>
 * An elevator user can be in one of four states as defined by {@link enumerations.UserState}.<br>
 * A user changes state when:
 * <ul> 
 * <li> it presses button(s) at a floor (see {@link #pressedButtonAtFloor()}), or </li>
 * <li> it is picked up at a floor (see {@link #pickedUpAtFloor()}), or </li>
 * <li> delivered to a floor (see {@link #deliveredToFloor()}).</li>
 * </ul>
 * The user is in a safe state when it respects the {@link #invariant()}.<br>
 * Each user has a unique integer identifier also used as the {@link #hashCode()}.<br>
 * @author jpaulgibson
 */
public interface ElevatorUserSpecification extends HasInvariant{
	
	
	/**
	 * State transition: ARRIVED_AT_FLOOR to WAITING_AT_FLOOR <br>
	 * Correct behaviour tested by {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_stateTransitions()}.<br>
	 * Exceptional behaviour tested by {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_stateTransitionsKO2()}
	 * and {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_stateTransitionsKO4()}.<br>
	 * @throws IllegalStateException if the user has not just arrived at the floor
	 */
	public void pressedButtonAtFloor() throws IllegalStateException;
	
	/**
	 * State transition: WAITING_AT_FLOOR to IN_ELEVATOR <br>
	 * Correct behaviour tested by {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_stateTransitions()}.<br>
	 * Exceptional behaviour tested by {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_stateTransitionsKO2()}
	 * and {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_stateTransitionsKO4()}.<br>
	 * @throws IllegalStateException if the user is not waiting at a floor
	 */
	public void pickedUpAtFloor() throws IllegalStateException;
	
	/**
	 * State transition: IN_ELEVATOR to SERVICED<br>
     * Correct behaviour tested by {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_stateTransitions()}.<br>
	 * Exceptional behaviour tested by {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_stateTransitionsKO1()}
	 * and {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_stateTransitionsKO3()}.<br>
	 * @throws IllegalStateException if the user is not inside the elevator
	 */
	public void deliveredToFloor() throws IllegalStateException;
	
	/**
	 * The elevator user can be in 1 of 3 states (see {@link enumerations.UserState})
	 * depending on whether their request has been fully/partially fulfilled<br>
	 * Tested (implicitly) by {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_stateTransitions()}.<br> 
	 * Accessor method 
	 * @return elevator state
	 */
	public UserState get_state();
	
	/**
	 * Tested (implicitly) by {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_SetUp}.<br> 
	 * Accessor method 
	 * @return destination floor
	 */
	public int get_to();
	
	/**
	 * Tested (implicitly) by {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_SetUp}.<br> 
	 * Accessor method 
	 * @return  floor on which request is made
	 */
	public int get_from();
	
	/**
	 * Based on the current state of the elevator and the user type, the user decides on how they behave<br>
	 * The different user types can be found in {@link enumerations.UserType}. <br>
	 * It is intended that the decision will be made dynamically based on the specific subclass (type) of the user at the 
	 * time of the call. <br>
	 * Each of the specific subclasses are found in the package {@link models.elevatorUsers}<br>
	 * The tests for correct behaviour are found in the package {@link unit_tests.elevatorUsers}<br>
	 */
	public void makeDecision();
	
	/**
	 * Tested by {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_toString()}.<br> 
	 * String output format as in the following example:
	 <pre>
	 ElevatorUserType 0 - from: 0 to: 1 currently WAITING_AT_FLOOR
	 </pre>
	 * @return the string representation in the specified format, 
	 */
	public String toString ();
	
	/**
	 * Tested by {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification}<br>
	 * <ul>
	 * <li> The user must want to go to a floor different from the one on which they are waiting for the elevator</li>
	 * <li> The floor on which the user is found cannot be below the bottom floor</li>
	 * <li> The floor to which the user wishes to go cannot be below the bottom floor</li>
	 * </ul>
	 */
	public boolean invariant();
	
	/**
	 * Tested by {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_equalsHash()}<br>
	 * @return the unique integer ID allocated to the user
	 */
	public int hashCode ();
	
	/**
	 * Tested by {@link unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification#test_equalsHash()}<br>
	 * @return true iff the 2 objects share the same UniqueID
	 */
	public boolean equals (Object obj);
    
}
