package models.elevatorButtons;

import tools.HasInvariant;
import unit_tests.buttons.JUnit_FloorButtonsInElevator;

/**
 * Tested by {@link unit_tests.buttons.JUnit_FloorButtonsInElevator} 
 * and {@link validation_tests.Validation_FloorButtonsInElevator}<br>
 * For managing the state of the floor requests inside the elevator
 * (see {@link interfaces.ElevatorSpecification})
 * @author J Paul Gibson
 */
public class FloorButtonsInElevator implements HasInvariant{
	
	/**
	 * The number of floors is in range <code> 0 .. NUMBER_OF_FLOORS</code><br>
	 * This must be no smaller than 2
	 */
	public final int NUMBER_OF_FLOORS;
	
	/**
	 * Inside the elevator there is an array of floor request buttons that can be pressed.
	 */
	protected boolean [] floorRequest;

	/**
	 * Initialises all floor requests (inside the elevator) to be false
	 * @param numberOfFloors is the number of floors in the lift system
	 * @throws IllegalArgumentException if the number of floors is smaller than 2
	 */
	public FloorButtonsInElevator (int numberOfFloors) throws IllegalArgumentException{
			
		if (numberOfFloors <2) 
			throw new IllegalArgumentException("The number of floors must be greater than 2");
		NUMBER_OF_FLOORS = numberOfFloors;	
		
		floorRequest = new boolean [NUMBER_OF_FLOORS];
		
		for (int floor =0; floor<NUMBER_OF_FLOORS;floor++){
			floorRequest [floor] = false;
		}
		
	}
	
	/**
	 * The following safety conditions are required:
	 * <ul>
	 * <li> number of floors must be at least 2 </li>
	 * <li> the number of floor buttons is the same as the number of floors</li>
	 * </ul>
	 * Tested by {@link JUnit_FloorButtonsInElevator#test_invariant()}
	 * @return true if all these safety conditions are met, and false otherwise. 
	 */
	public boolean invariant(){
		
		if (NUMBER_OF_FLOORS<2) return false;
		if (floorRequest.length != NUMBER_OF_FLOORS) return false;
		return true;
	}
	
	/**
	 * Tested by {@link JUnit_FloorButtonsInElevator#test_presses_cancels()}
	 * @param floor is specifying the floor to check
	 * @return Whether there is currently a request to go to the specified floor (inside the elevator)
	 * @throws IllegalArgumentException if the number of floors specified is not in range
	 */
	public boolean floorPressedInElevator (int floor) throws IllegalArgumentException{
			
		if (floor<0 || floor >= NUMBER_OF_FLOORS) 
			throw new IllegalArgumentException("The number of floors specified is not in range");
		return floorRequest[floor];}
	
	/**
	 * Tested by {@link JUnit_FloorButtonsInElevator#test_presses_cancels()}
	 * Update the state when a request is made for the specified floor (inside the elevator)
	 * @param floor is specifying the floor
     * @throws IllegalArgumentException if the number of floors specified is not in range
	 */
	public void pressFloorInElevator (int floor)throws IllegalArgumentException{
			
		if (floor<0 || floor >= NUMBER_OF_FLOORS) 
			throw new IllegalArgumentException("The number of floors specified is not in range");
		floorRequest[floor]= true;}

	/**
	 * Tested by {@link JUnit_FloorButtonsInElevator#test_presses_cancels()}
	 * Update the state when a request for the specified floor has been served
	 * @param floor is specifying the floor to check
	 * @throws IllegalArgumentException if the number of floors specified is not in range
	 */
	public void cancelFloorRequestInElevator (int floor)throws IllegalArgumentException{
			
		if (floor<0 || floor >= NUMBER_OF_FLOORS) 
			throw new IllegalArgumentException("The number of floors specified is not in range");
		floorRequest[floor]= false;}
	
	/**
	 * Tested by {@link JUnit_FloorButtonsInElevator#test_noMoreRequestsAbove()}
	 * @param floor is specifying the floor to check
	 * @return whether there is a request for a floor (inside the elevator) above the specified floor
     * @throws IllegalArgumentException if the number of floors specified is not in range
	 */
	public boolean noMoreRequestsAbove(int floor)throws IllegalArgumentException{
			
		if (floor<0 || floor >= NUMBER_OF_FLOORS) 
			throw new IllegalArgumentException("The number of floors specified is not in range");
		
		for (int i = floor+1; i < NUMBER_OF_FLOORS; i++){ if (floorRequest[i]) return false;}
		
		return true;
	}
	
	/**
     * Tested by {@link JUnit_FloorButtonsInElevator#test_noMoreRequestsBelow()}
   	 * @param floor is specifying the floor to check
	 * @return whether there is a request for a floor (inside the elevator) above the specified floor
     * @throws IllegalArgumentException if the number of floors specified is not in range

	 */
	public boolean noMoreRequestsBelow(int floor)throws IllegalArgumentException{
			
		if (floor<0 || floor >= NUMBER_OF_FLOORS) 
			throw new IllegalArgumentException("The number of floors specified is not in range");
		
		for (int i = floor-1; i >=0 ; i--){ 
			if (floorRequest[i]) return false;
		}
		
		return true;
	}
	
	/**
	 * Tested by {@link JUnit_FloorButtonsInElevator#test_toString()}<br>
	 * Lists the floors that have been requested inside the elevator<br>
	 * Typical output is of the form: <br>
	 * <pre>
	 * Requested Floors Inside Elevator: 2 4 7
	 * </pre>
	 * In this example, the floors requested (2,4 and 7) are listed in order and separated by single spaces
	 */
	public String toString(){
		String str="Requested Floors Inside Elevator:";
		for (int floor=0; floor< NUMBER_OF_FLOORS;floor++)
			if (floorRequest[floor]) str = str + " "+floor;
		return str;
		
	}
}
