package models.elevatorLogicUnit;

import abstractions.LogcUnitForElevatorBinding;
import enumerations.LogicType;
import enumerations.DirectionState;
import interfaces.ElevatorForLogicUnitSpecification;

/**
 * The smart controller ensures that the elevator will change direction if there are
 * no more requests in the current direction and their are requests in the opposite direction.<br>
 * The elevator stops only at floors where requests to stop have been made.<br>
 * This corresponds to SMART in the {@link enumerations.LogicType}
 * @author J Paul Gibson
 */
public class SmartLogicUnit extends LogcUnitForElevatorBinding{

	
	public SmartLogicUnit(ElevatorForLogicUnitSpecification elevator) {
		super(elevator);
	}
	
	/**
	 * Only stop at floors where requests to stop have been made
	 */
	public boolean stopAtNextFloor(){
		
		int floor = elevator.currentFloor();
		DirectionState dir = elevator.currentDirection();
		
		if (elevator.floorPressedInElevator(floor)) return true;
		if (elevator.downPressedAtFloor(floor) && dir == DirectionState.DOWN) return true;
		if (elevator.upPressedAtFloor(floor)  && dir == DirectionState.UP) return true;
		
		return false;
	}
	


	@Override
	public DirectionState calculateDirection() {
		
		int currentFloor = elevator.currentFloor();
		DirectionState currentDirection = elevator.currentDirection();
		boolean noMoreRequestsAboveCurrent = elevator.noMoreRequestsAbove(currentFloor);
		boolean noMoreRequestsBelowCurrent = elevator.noMoreRequestsBelow(currentFloor);
		boolean floorDownRequestAtCurrent = elevator.downPressedAtFloor(currentFloor);
		boolean floorUpRequestAtCurrent = elevator.upPressedAtFloor(currentFloor);
		
		
		if (currentDirection == DirectionState.STAY){
			
			if (noMoreRequestsAboveCurrent  && noMoreRequestsBelowCurrent) return DirectionState.STAY;
			else if (noMoreRequestsAboveCurrent) return DirectionState.DOWN;
			else return DirectionState.UP;
		}
		
		else if (currentDirection == DirectionState.UP){

			if (noMoreRequestsAboveCurrent  && noMoreRequestsBelowCurrent){
				
				 if (!floorDownRequestAtCurrent	&& !floorUpRequestAtCurrent) return DirectionState.STAY;
				 if (floorUpRequestAtCurrent) return DirectionState.UP;
				 else return  DirectionState.DOWN;
			
			}
					
			else if (noMoreRequestsAboveCurrent) return DirectionState.DOWN;
			else return DirectionState.UP;
		}
		
		else if (currentDirection == DirectionState.DOWN){
			

			if (noMoreRequestsAboveCurrent  && noMoreRequestsBelowCurrent){
				 if (!floorDownRequestAtCurrent	&& !floorUpRequestAtCurrent) return DirectionState.STAY;
				 if (floorDownRequestAtCurrent) return DirectionState.DOWN;
				 else return  DirectionState.UP;
			
			}
			else if (noMoreRequestsBelowCurrent) return DirectionState.UP;
			else return DirectionState.DOWN;
		}
		
		   return DirectionState.STAY; // should not happen
	}
	
	public String toString (){
		
		return LogicType.SMART.toString();
	}
}
