package simulations;

import interfaces.ElevatorObserverSpecification;
import interfaces.ElevatorSpecification;
import interfaces.ElevatorUserSpecification;

import java.util.ArrayList;
import java.util.Random;

import tools.Ticker;
import tools.TickerObserverSpecification;
import enumerations.UserType;
import factoriesSPL.ElevatorUserFactory;

/**
 * Tested by {@link validation_tests.Validation_Users_Simulation}.<br>
 * A generator for random elevator users which executes in an independent thread which can be turned on/off.<br>
 * If the users are linked to an observable elevator then when the elevator changes state
 * the {@link #elevatorUpdated()} method is executed in order to prompt each user to make a decision concerning
 * their behaviour.
 * @author jpaulgibson
 *
 */
public class Users_Simulation implements ElevatorObserverSpecification, TickerObserverSpecification{
	
	  /**
	   * Use to turn on/off writing to console/terminal when state updates occur<br>
	   * This is <code> false </code>  by default, as the simulation is to be used in a GUI without the use of a terminal<br>
	   * It is set to <code> true </code> for the validation tests defined in {@link validation_tests.Validation_Users_Simulation}.
	   */
	  boolean terminal_output_switch = false;
	  
	  
	  /**
	   * The ticker that is used to add new users to the elevator system
	   */
	  final Ticker TICKER_FOR_ADDING_USERS;
	
	 /**
	  * The elevator to be used in the simulation - all users being generated are associated with this 
	  * single, common elevator.
	  */
	 final ElevatorSpecification ELEVATOR;
	 
	 /**
	  * The RNG to be used during the simulation for users with randomly generated journey requirements
	  * for moving between floors.
	  */
	 final Random RNG;

	/**
	 * The time between ticks of the internal thread timer, which corresponds to the frequency with 
	 * which new users are created
	 */
	private final int DELAY_TIME;
	
	 /**
	  * The type of user to use in the simulation
	  */
	 final UserType USER_TYPE;
	 
	 /**
	  * The users of the lift (who arrive randomly during the simulation).<br>
	  * The number of users is not known in advance - it depends on the length of the simulation - so we use
	  * a collection which grows automatically.<br>
	  * We do not need to worry about multiple threads accessing this list so we use an ArrayList and not a Vector.
	  */
	 final  ArrayList<ElevatorUserSpecification> USERS;
	 
	 
	 final static ElevatorUserFactory userFactory = ElevatorUserFactory.getInstance();
   
  /**
   * Constructor for making a thread that will generate random elevator users periodicially.<br>
   * We do not need to simulate a more realistic distribution of elevator users (which would be much more complicated) as our
   * simple simulation is to be used only to validate - through a graphical animation - the behaviour of the elevator.
   * @param rng is the random number generated to be used to make the random users
   * @param elevator is the elevator common to all users, and which will be  observed by the simulation thread 
   * @param delayTime is the time between the generation of each user
   * @param userType is the type of elevator user
   * @see enumerations.UserType
   */
  public Users_Simulation (Random rng, ElevatorSpecification  elevator, int delayTime, UserType userType){
	   
	   this.ELEVATOR = elevator;
	   this.RNG = rng;
	   this.DELAY_TIME = delayTime;
	   this.USER_TYPE = userType;
	   this.TICKER_FOR_ADDING_USERS = new Ticker(DELAY_TIME);
	   this.USERS = new ArrayList<ElevatorUserSpecification>();
	   elevator.connect_Observer(this);	 
       TICKER_FOR_ADDING_USERS.connect_Observer(this);
  }



	/**
	 * All users are required to make a decision concerning whether the elevator's state change requires them
	 * to act in some way.
	 */
	@Override
	public void elevatorUpdated() {		
		for (ElevatorUserSpecification user: USERS)  		 
 		   user.makeDecision();
	     if (terminal_output_switch) 
	 	         System.out.println("USERS: "+ USERS); 
		
	}

	@Override
	public void observe_ticker() {
		
		 ElevatorUserSpecification user = userFactory.makeElevatorUser(USER_TYPE, RNG, ELEVATOR);
         USERS.add(user);
         
         if (terminal_output_switch) 
         System.out.println("Added new user: "+ user);
	}

	public void animation_on(){
		TICKER_FOR_ADDING_USERS.animation_on();
	}
	
	public void animation_off(){
		TICKER_FOR_ADDING_USERS.animation_off();
	}
	
	
	public String toString () {
		String str = "Users_Simulation: ";
				for (ElevatorUserSpecification user: USERS)		 
			 		  str = str +  user.toString();
		return str;
	}
	
	public void terminal_output(boolean on){
		terminal_output_switch = on;
	}

}
