package validation_tests;

import interfaces.ElevatorSpecification;

import java.util.Random;

import enumerations.UserType;
import factoriesSPL.ElevatorFactory;
import simulations.Users_Simulation;
import tools.DateHeader;
import tools.SeedRNGCommandLine;

/**
 * Validation test for {@link Users_Simulation} <br>
 * The following needs to be checked:
 * <ul> 
 * <li> The users arrive at regular intervals, in order in which they are create, separated by a short delay</li>
 * <li> The users have their journeys randomly chosen (fairly) </li>
 * <li> The users are all initially waiting at the floor from which they wish to travel </li>
 * <li> All users wish to make valid journeys </li>
 * </ul>
 * Note that the type of elevator has no impact on the behaviour being tested 
 * (except for the number of floors being serviced), so we use the default constructor.<br>
 * 
 * The behaviour is random (the seed value can be provided as a command line parameter).<br>
 * Typical output:
 <pre>
The seed used for the random number generator in the test is 0.
You can override this value by passing an integer value as a main argument parameter, if you so wish.


********************************************************************
Execution Date/Time 2015/07/28 09:28:35
********************************************************************
Users Simulation Started
Added new user: IntelligentElevatorUser 0 - from: 2 to: 3 currently WAITING_AT_FLOOR
Added new user: IntelligentElevatorUser 1 - from: 0 to: 2 currently WAITING_AT_FLOOR
Added new user: IntelligentElevatorUser 2 - from: 2 to: 1 currently WAITING_AT_FLOOR
Added new user: IntelligentElevatorUser 3 - from: 2 to: 0 currently WAITING_AT_FLOOR
Users Simulation Finished
Simulation time (ms) = 4002
</pre>
 * @author jpaulgibson
 */
public class Validation_Users_Simulation {
	
	 /**
	  * Simulation time 
	  */
	 final static int SIMULATION_TIME = 4000; //ms
	 
	 /**
	  * NUmber of random users to generate during the simulation
	  */
	 final static int NUMBER_OF_USERS = 4;
	 
	/**
	  * Frequency at which new users arrive
	  */
	 final static int USER_ARRIVAL_FREQUENCY = SIMULATION_TIME / (NUMBER_OF_USERS+1); // ms
	 
	 /**
	  * The type of user to use in the simulation
	  */
	 final static UserType USER_TYPE = UserType.INTELLIGENT;
 	 
	
    
    /**
     * Doors on the floors?
     */
    final static boolean DOORS = false;
	
    public static void main(String[] args) {
	
		 Random rng = SeedRNGCommandLine.getRandom(args);
	     System.out.println(DateHeader.dateString());
	     
	     ElevatorSpecification elevator = 
			ElevatorFactory.makeElevator();
  	 
         Users_Simulation usersSimulation = new Users_Simulation(rng, elevator, USER_ARRIVAL_FREQUENCY, USER_TYPE);

         usersSimulation.terminal_output(true); // Switch on output to screen
         
         System.out.println("Users Simulation Started");
  		 usersSimulation.animation_on();
  		 
  		long startTime = System.currentTimeMillis();
 		
 		try { Thread.sleep(SIMULATION_TIME);
 		    } catch (InterruptedException e) {e.printStackTrace();}
 		
 		usersSimulation.animation_off();

 	    System.out.println("Users Simulation Finished");
 	    
 	   long stopTime = System.currentTimeMillis();
       long elapsedTime = stopTime - startTime;
       System.out.println("Simulation time (ms) = "+ elapsedTime);
	}

}
